@@ -3,7 +3,7 @@ from dotenv import load_dotenv
 load_dotenv()
 from discord_webhook import AsyncDiscordWebhook ,DiscordEmbed
 import pandas as pd
-from apis.polygonio.models.option_models.universal_snapshot import OptionData
+
 
 from apis.polygonio.mapping import option_condition_desc_dict, stock_condition_desc_dict
 from _markets.list_sets.dicts import hex_color_dict as hex_colors
@@ -1042,16 +1042,8 @@ async def profit_ratio_02_embed(profit_ratio,stock_symbol, price, hex_colors, o,
 
 
 from typing import List
-async def poly_data_embed(title, description, color, hook, data_dict: OptionData):
 
 
-    webhook = AsyncDiscordWebhook(hook)
-    embed = DiscordEmbed(title=f"{title}", description=f"```py\n{description}```", color=color)
-
-    webhook.add_embed(embed)
-
-    await webhook.execute()
-
 
 
 async def specials_embed_2(underlying_symbol, strike, call_put, expiry, buyPct,nPct,sellPct,official_open,last_price,day_vwap,buyVolume,neutralVolume,sellVolume,volume_percent_total, volume, avgPrice, totalNum, moneyness, price_diff, price_change, volume_to_price, buy_sell_vol_str, deriv_id):
@@ -1176,22 +1168,6 @@ async def theta_resistant_embed(data:dict):
 
 
 
-async def sector_dark_pool_embed(logo, ticker, webhook_url, sector, dollar_cost, conditions, exchange, size, price, volume_indicator, trade_timestamp):
-
-    hook = AsyncDiscordWebhook(webhook_url, content=f"<@375862240601047070>")
-    embed = DiscordEmbed(title=f"SECTOR DARK POOLS - {sector} | {ticker}", description=f"```py\n> Size: **{size}**\n> Price: **${price}**\n> Dollar Cost: **${round(float(dollar_cost),2)}**```\n> **These feeds are SECTOR DARK POOLS - Or Trades occuring out of the FINRA ADF/TRF Facility. Trades with dollar value of $1,000,000 or more will post here out of these exchanges.", color=market.colors.get('gold'))
-    embed.add_embed_field(name=f"Conditions:", value=f"> **{conditions}**")
-    embed.add_embed_field(name=f"Exchange:", value=f"> **{exchange}**")
-    embed.add_embed_field(name=f"Volume Indicator:", value=f"> **{volume_indicator}**")
-    embed.set_thumbnail(logo)
-    embed.set_timestamp()
-    embed.set_footer(text=f'{ticker} | {trade_timestamp}', icon_url=os.environ.get('fudstop_logo'))
-
-    hook.add_embed(embed)
-    asyncio.create_task(hook.execute())
-
-
-
 async def conditional_embed(webhook, row, color, description, status, sector, db):
     hook = AsyncDiscordWebhook(webhook)
 
