@@ -437,86 +437,89 @@ class DbManager:
 
 
     async def batch_insert_dataframe(self, df, table_name, unique_columns, batch_size=250):
-        async with lock:
-            if not await self.table_exists(table_name):
-                await self.create_table(df, table_name, unique_columns)
+        try:
+            async with lock:
+                if not await self.table_exists(table_name):
+                    await self.create_table(df, table_name, unique_columns)
 
-            # Debug: Print DataFrame columns before modifications
-            #print("Initial DataFrame columns:", df.columns.tolist())
-            
-            df = df.copy()
-            df.dropna(inplace=True)
-            df['insertion_timestamp'] = [datetime.now() for _ in range(len(df))]
+                # Debug: Print DataFrame columns before modifications
+                #print("Initial DataFrame columns:", df.columns.tolist())
+                
+                df = df.copy()
+                df.dropna(inplace=True)
+                df['insertion_timestamp'] = [datetime.now() for _ in range(len(df))]
 
-            # Debug: Print DataFrame columns after modifications
-            #print("Modified DataFrame columns:", df.columns.tolist())
-            
-            records = df.to_records(index=False)
-            data = list(records)
+                # Debug: Print DataFrame columns after modifications
+                #print("Modified DataFrame columns:", df.columns.tolist())
+                
+                records = df.to_records(index=False)
+                data = list(records)
 
 
-            async with self.pool.acquire() as connection:
-                column_types = await connection.fetch(
-                    f"SELECT column_name, data_type FROM information_schema.columns WHERE table_name = '{table_name}'"
-                )
-                type_mapping = {col: next((item['data_type'] for item in column_types if item['column_name'] == col), None) for col in df.columns}
+                async with self.pool.acquire() as connection:
+                    column_types = await connection.fetch(
+                        f"SELECT column_name, data_type FROM information_schema.columns WHERE table_name = '{table_name}'"
+                    )
+                    type_mapping = {col: next((item['data_type'] for item in column_types if item['column_name'] == col), None) for col in df.columns}
 
-                async with connection.transaction():
-                    insert_query = f"""
-                    INSERT INTO {table_name} ({', '.join(f'"{col}"' for col in df.columns)}) 
-                    VALUES ({', '.join('$' + str(i) for i in range(1, len(df.columns) + 1))})
-                    ON CONFLICT ({unique_columns})
-                    DO UPDATE SET {', '.join(f'"{col}" = excluded."{col}"' for col in df.columns)}
-                    """
-            
-                    batch_data = []
-                    for record in data:
-                        new_record = []
-                        for col, val in zip(df.columns, record):
-                            pg_type = type_mapping[col]
+                    async with connection.transaction():
+                        insert_query = f"""
+                        INSERT INTO {table_name} ({', '.join(f'"{col}"' for col in df.columns)}) 
+                        VALUES ({', '.join('$' + str(i) for i in range(1, len(df.columns) + 1))})
+                        ON CONFLICT ({unique_columns})
+                        DO UPDATE SET {', '.join(f'"{col}" = excluded."{col}"' for col in df.columns)}
+                        """
+                
+                        batch_data = []
+                        for record in data:
+                            new_record = []
+                            for col, val in zip(df.columns, record):
+                                pg_type = type_mapping[col]
 
-                            if val is None:
-                                new_record.append(None)
-                            elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone']:
-                                if isinstance(val, np.datetime64):
-                                    # Convert numpy datetime64 to Python datetime, ensure UTC and remove tzinfo if needed
-                                    new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
-                                elif isinstance(val, datetime):
-                                    # Directly use the Python datetime object
+                                if val is None:
+                                    new_record.append(None)
+                                elif pg_type in ['timestamp', 'timestamp without time zone', 'timestamp with time zone']:
+                                    if isinstance(val, np.datetime64):
+                                        # Convert numpy datetime64 to Python datetime, ensure UTC and remove tzinfo if needed
+                                        new_record.append(pd.Timestamp(val).to_pydatetime().replace(tzinfo=None))
+                                    elif isinstance(val, datetime):
+                                        # Directly use the Python datetime object
+                                        new_record.append(val)
+                                elif pg_type in ['double precision', 'real'] and not isinstance(val, str):
+                                    new_record.append(float(val))
+                                elif isinstance(val, np.int64):
+                                    new_record.append(int(val))
+                                elif pg_type == 'integer' and not isinstance(val, int):
+                                    new_record.append(int(val))
+                                else:
                                     new_record.append(val)
-                            elif pg_type in ['double precision', 'real'] and not isinstance(val, str):
-                                new_record.append(float(val))
-                            elif isinstance(val, np.int64):
-                                new_record.append(int(val))
-                            elif pg_type == 'integer' and not isinstance(val, int):
-                                new_record.append(int(val))
-                            else:
-                                new_record.append(val)
-                        batch_data.append(new_record)
+                            batch_data.append(new_record)
 
-                        if len(batch_data) == batch_size:
-                            try:
+                            if len(batch_data) == batch_size:
+                                try:
+                                    
                                 
-                             
-                                await connection.executemany(insert_query, batch_data)
-                                batch_data.clear()
-                            except Exception as e:
-                                print(f"An error occurred while inserting the record: {e}")
-                                await connection.execute('ROLLBACK')
-                                raise
+                                    await connection.executemany(insert_query, batch_data)
+                                    batch_data.clear()
+                                except Exception as e:
+                                    print(f"An error occurred while inserting the record: {e}")
+                                    await connection.execute('ROLLBACK')
+                                    raise
 
-                if batch_data:  # Don't forget the last batch
-       
-                    try:
+                    if batch_data:  # Don't forget the last batch
+        
+                        try:
 
-                        await connection.executemany(insert_query, batch_data)
-                    except Exception as e:
-                        print(f"An error occurred while inserting the record: {e}")
-                        await connection.execute('ROLLBACK')
-                        raise
+                            await connection.executemany(insert_query, batch_data)
+                        except Exception as e:
+                            print(f"An error occurred while inserting the record: {e}")
+                            await connection.execute('ROLLBACK')
+                            raise
+        except Exception as e:
+            print(e)
 
                     
-    async def save_to_history(self, df, main_table_name, history_table_name):
+    async def save_to_history(self, df, history_table_name):
         # Assume the DataFrame `df` contains the records to be archived
         if not await self.table_exists(history_table_name):
             await self.create_table(df, history_table_name, None)
