@@ -13,7 +13,7 @@ from list_sets.ticker_lists import most_active_tickers
 import asyncio
 
 
-db = PolygonOptions(user='chuck', database='markets')
+db = PolygonOptions(user='chuck', database='market_data')
 async def main(ticker):
     
     all_options = await db.get_option_chain_all(underlying_asset=ticker)
