@@ -33,13 +33,13 @@ class StockMarketLive:
         self.pool = None
         
         self.connection_string = os.environ.get('STOCK_MARKET')  # Use one environment variable for connection string
-        self.polygon = Polygon(host='localhost', user='chuck', password='fud', database='markets', port=5432)
+        self.polygon = Polygon(host='localhost', user='chuck', password='fud', database='market_data', port=5432)
         self.timespans = ['minute', 'hour', 'day', 'week', 'month']
 
         self.c = WebSocketClient(subscriptions=["T.*"], custom_json=orjson, api_key=os.environ.get('YOUR_POLYGON_KEY'))
 
     async def connect(self):
-        self.pool = await asyncpg.create_pool(host='localhost', user='chuck', database='markets', port=5432, password='fud', min_size=1, max_size=10)
+        self.pool = await asyncpg.create_pool(host='localhost', user='chuck', database='market_data', port=5432, password='fud', min_size=1, max_size=10)
         return self.pool
     async def fetch(self, query):
         async with self.pool.acquire() as conn:
