@@ -11,15 +11,15 @@ from io import StringIO
 import pandas as pd
 from .models import AuctionResult, FXSwaps, TimeSeries, AsOfDates, TimeSeriesData, SecuredReferenceRates, RepoOperations, SecuritiesLending
 session = requests.session()
-
-today_str = '2023-10/29'
-
+from datetime import datetime, timedelta
+today_str = datetime.now().strftime('%Y-%m-%d')
+thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
 class FedNewyork:
     def __init__(self):
         self.base_url = "https://markets.newyorkfed.org/api/"
-
-
-    def agency_mbs_search(self, start_date="2023-01-01", end_date=today_str):
+        self.thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
+        self.today = datetime.now().strftime('%Y-%m-%d')
+    def agency_mbs_search(self, start_date=thirty_days_ago, end_date=today_str):
         """Search for AMBS operations out of the Federal Reserve of New York"""
 
         r = session.get(self.base_url + f"ambs/all/results/summary/search.json?startDate={start_date}&endDate={today_str}").json()
@@ -117,18 +117,18 @@ class FedNewyork:
             return None
         
     def get_timeseries(self):
-            """Returns the timeseries data to query the FED API"""
+        """Returns the timeseries data to query the FED API"""
 
-            r = requests.get("https://markets.newyorkfed.org/api/pd/list/timeseries.json").json()
-            pdd = r['pd']
-            timeseries = pdd['timeseries']
-            if timeseries is not None:
-                data = TimeSeries(timeseries)
+        r = requests.get("https://markets.newyorkfed.org/api/pd/list/timeseries.json").json()
+        pdd = r['pd']
+        timeseries = pdd['timeseries']
+        if timeseries is not None:
+            data = TimeSeries(timeseries)
 
-                
-                return data
-            else:
-                return None
+            
+            return data
+        else:
+            return None
 
 
     def get_timeseries_data(self, timeseries):
@@ -161,8 +161,12 @@ class FedNewyork:
 
         
 
-    def rates_search(self, start_date="2023-01-01", end_date=today_str):
+    def rates_search(self, start_date:str=None, end_date:str=None):
         """Search reference rates between a given time range."""
+        if start_date == None:
+            start_date = self.thirty_days_ago
+        if end_date == None:
+            end_date = self.today
         r = session.get(self.base_url + f"rates/all/search.json?startDate={start_date}&endDate={end_date}").json()
         refrates = r['refRates']
         if refrates is not None:
@@ -172,9 +176,10 @@ class FedNewyork:
             return None
         
 
-    def repo_operations_search(self, start_date="2023-01-01", end_date=today_str):
+    def repo_operations_search(self, start_date=None, end_date=today_str):
         """Search by date for repo operations out of the FED."""
-
+        if start_date == None:
+            start_date = self.thirty_days_ago
         r = session.get(f"https://markets.newyorkfed.org/api/rp/results/search.json?startDate={start_date}&endDate={end_date}&securityType=mb").json()
         repo = r['repo']
         operations = repo['operations']
@@ -226,8 +231,10 @@ class FedNewyork:
         return df
 
 
-    def securities_lending_search(self, start_date="2023-01-01", end_date=today_str):
+    def securities_lending_search(self, start_date=None, end_date=today_str):
         """Search securities lending operations out of the FED."""
+        if start_date == None:
+            start_date = self.thirty_days_ago
         sec_lending = session.get(f"https://markets.newyorkfed.org/api/seclending/all/results/summary/search.json?startDate={start_date}&endDate={end_date}").json()
 
         seclending = sec_lending['seclending']
@@ -275,9 +282,9 @@ class FedNewyork:
             'xlsx'
         
         """
-        url = f"https://markets.newyorkfed.org/beta/api/ambs/all/results/details/search.json?startDate=2023-10-01&endDate={today_str}"
+        url = f"https://markets.newyorkfed.org/beta/api/ambs/all/results/details/search.json?startDate={self.thirty_days_ago}&endDate={today_str}"
         print(url)
-        r = requests.get(url, headers=self.headers).json()
+        r = requests.get(url).json()
         ambs = r['ambs'] if 'ambs' in r else None
 
         all_data_dicts = []
@@ -328,20 +335,20 @@ class FedNewyork:
         if seclending is not None:
             operations = seclending['operations'] if 'operations' in seclending else None
             if operations is not None:
-                return operations
+                df = pd.DataFrame(operations)
+
+                return df
 
     def treasury_holdings(self):
         url = f"https://markets.newyorkfed.org/api/tsy/all/results/summary/last/10.json"
         r = requests.get(url).json()
         treasury = r['treasury']
         auctions = treasury['auctions'] if 'auctions' in treasury else None
-        if auctions is not None:
-            return auctions
-        
-
+        df = pd.DataFrame(auctions)
+        return df
     def data_act_compliance(self):
         base_url = "https://api.fiscaldata.treasury.gov/services/api/fiscal_service/"
-        url=base_url+f"/v2/debt/tror/data_act_compliance?filter=record_date:gte:2023-07-01,record_date:lte:{today_str}&sort=-record_date,agency_nm,agency_bureau_indicator,bureau_nm"
+        url=base_url+f"/v2/debt/tror/data_act_compliance?filter=record_date:gte:{self.thirty_days_ago},record_date:lte:{today_str}&sort=-record_date,agency_nm,agency_bureau_indicator,bureau_nm"
         r = requests.get(url).json()
         data = r['data']
         df = pd.DataFrame(data)
@@ -354,9 +361,9 @@ class FedNewyork:
         r = requests.get(url).json()
         soma = r['soma']
         summary = soma['summary'] if 'summary' in soma else None
-        if summary is not None:
-            return summary
-        
+
+        df = pd.DataFrame(summary)
+        return df        
 
     def market_share(self):
         url="https://markets.newyorkfed.org/api/marketshare/qtrly/latest.json"
@@ -463,10 +470,11 @@ class FedNewyork:
             }
             data_dicts.append(data_dict)
 
-        return data_dicts
+        df = pd.DataFrame(data_dicts)
+        return df
 
 
-    def central_bank_liquidity_swaps():
+    def central_bank_liquidity_swaps(self):
         """Returns operations out of the fed for central bank liquidity swaps
         
         ARGS:
@@ -478,7 +486,7 @@ class FedNewyork:
         
         """
 
-        url = f"https://markets.newyorkfed.org/api/fxs/usdollar/last/5.json"
+        url = f"https://markets.newyorkfed.org/api/fxs/usdollar/last/10.json"
 
         r = requests.get(url).json()
         fxswaps = r["fxSwaps"]
@@ -515,7 +523,7 @@ class FedNewyork:
 
 
 
-    def reverse_repo():
+    def reverse_repo(self):
         url = f"https://markets.newyorkfed.org/api/rp/all/all/results/last/10.json"
         r = requests.get(url)
         r.raise_for_status()  # This will raise an exception if the request failed
@@ -561,5 +569,5 @@ class FedNewyork:
                 }}
 
                 all_operations.append(op_data)
-
-        return all_operations
\ No newline at end of file
+        df = pd.DataFrame(all_operations)
+        return df, RepoOperations(all_operations)
\ No newline at end of file
