@@ -127,40 +127,31 @@ class Polygon:
         except Exception:
             print(f"ERROR!")
             
-    async def paginate_concurrent(self, url, as_dataframe=False, concurrency=250):
-
+    async def paginate_concurrent(self, url, as_dataframe=False, concurrency=250, filter:bool=False):
         all_results = []
-
-        
-
         pages_to_fetch = [url]
-        
-        while pages_to_fetch:
-            tasks = []
-            
-            for _ in range(min(concurrency, len(pages_to_fetch))):
-                next_url = pages_to_fetch.pop(0)
-                tasks.append(self.fetch_page(next_url))
-                
-            results = await asyncio.gather(*tasks)
-            if results is not None:
-                for data in results:
-                    if data is not None:
-                        if "results" in data:
-                            all_results.extend(data["results"])
 
-                            
+        async with httpx.AsyncClient() as session:
+            while pages_to_fetch:
+                tasks = []
+                current_urls = pages_to_fetch[:concurrency]
+                pages_to_fetch = pages_to_fetch[concurrency:]  # Update remaining pages
+
+                for next_url in current_urls:
+                    tasks.append(self.fetch_page(next_url))
+
+                results = await asyncio.gather(*tasks)
+                for data in results:
+                    if data is not None and "results" in data:
+                        all_results.extend(data["results"])
                         next_url = data.get("next_url")
                         if next_url:
-                            next_url += f'&{urlencode({"apiKey": f"{self.api_key}"})}'
+                            next_url += f'&{urlencode({"apiKey": os.environ.get("YOUR_POLYGON_KEY")})}'
                             pages_to_fetch.append(next_url)
-                    else:
-                        break
+
         if as_dataframe:
             import pandas as pd
             return pd.DataFrame(all_results)
-        else:
-            return all_results
         
 
     async def fetch_endpoint(self, url):
@@ -722,42 +713,6 @@ class Polygon:
 
 
 
-    async def get_universal_snapshot(self, ticker, retries=3): #✅
-        """Fetches the Polygon.io universal snapshot API endpoint"""
-        timeout = aiohttp.ClientTimeout(total=10)  # 10 seconds timeout for the request
-        
-        for retry in range(retries):
-        # async with sema:
-            url = f"https://api.polygon.io/v3/snapshot?ticker.any_of={ticker}&apiKey={self.api_key}&limit=250"
-            await self.create_session()
-            try:
-            
-
-                try:
-                    async with self.session.get(url) as resp:
-                        data = await resp.json()
-                        results = data.get('results', None)
-        
-                        if results is not None:
-                            flattened_results = [flatten_dict(result) for result in results]
-                            return flattened_results
-                            
-                except aiohttp.ClientConnectorError:
-                    print("ClientConnectorError occurred. Retrying...")
-                    continue
-                
-                except aiohttp.ContentTypeError as e:
-                    print(f"ContentTypeError occurred: {e}")  # Consider logging this
-                    continue
-                
-                except Exception as e:
-                    print(f"An unexpected error occurred: {e}")  # Consider logging this
-                    continue
-            except Exception as e:
-                print(e)
-
-
-
     async def get_price(self, ticker):
         try:
             if ticker in ['SPX', 'NDX', 'XSP', 'RUT', 'VIX']:
@@ -974,15 +929,9 @@ class Polygon:
         # Define the endpoint without query parameters
         endpoint = f"https://api.polygon.io/v3/trades/{ticker}?timestamp.gte={timestamp_gte}&timestamp.lte={timestamp_lte}&apiKey={self.api_key}&limit={limit}"
 
-        await self.create_session()
+        data = await self.paginate_concurrent(endpoint)
 
-        async with self.session.get(endpoint) as response:
-            data = await response.json()
-            results = data.get('results')
-            if results is not None:
-                return TradeData(results, ticker)
-            else:
-                print(f'No data for {ticker}')
+        return TradeData(data, ticker=ticker)
             
 
     async def test_trades(self, ticker: str, limit: int = 50000, timestamp_gte: str = None, timestamp_lte: str = None):
@@ -1016,18 +965,3 @@ class Polygon:
             await self.close_session()
 
 
-
-    async def get_universal_snapshots_by_type(self, type:str='stocks'):
-        try:
-            await self.create_session()
-
-            endpoint = f"https://api.polygon.io/v3/snapshot?type={type}&apiKey={self.api_key}"
-
-            data = await self.fetch_page(endpoint)
-            if data is not None:
-                results = data.get('results')
-                if results is not None:
-                    return results
-
-        finally:
-            await self.close_session()
\ No newline at end of file
