@@ -22,7 +22,7 @@ from datetime import datetime, timedelta
 
 
 import numpy as np
-from .models.option_models.universal_snapshot import UniversalOptionSnapshot, UniversalSnapshot, OptionData, UniversalOptionSnapshot2
+from .models.option_models.universal_snapshot import UniversalOptionSnapshot,UniversalOptionSnapshot2, SpxSnapshot
 from .polygon_helpers import flatten_nested_dict, flatten_dict
 lock = Lock()
 
@@ -1348,20 +1348,16 @@ class PolygonOptions:
 
     async def get_universal_snapshot(self, ticker): #✅
         """Fetches the Polygon.io universal snapshot API endpoint"""
-        url=f"https://api.polygon.io/v3/snapshot?ticker.any_of={ticker}&limit=250&apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+        url=f"https://api.polygon.io/v3/snapshot?ticker.any_of={ticker}&limit=250&apiKey={self.api_key}"
         print(url)
   
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.get(url) as resp:
-                    data = await resp.json()
-                    results = data['results'] if 'results' in data else None
-                    if results is not None:
-                        return UniversalOptionSnapshot(results)
-                    else:
-                        return None
-            except ContentTypeError:
-                pass
+        async with httpx.AsyncClient() as client:
+            resp = await client.get(url)
+            data = resp.json()
+            results = data['results'] if 'results' in data else None
+            if results is not None:
+                return SpxSnapshot(results)
+
     async def get_option_chain_all(self, 
                                     underlying_asset: str, 
                                     strike_price: float = None, 
@@ -1426,11 +1422,12 @@ class PolygonOptions:
 
                 # Construct the API endpoint and query string
                 endpoint = f"https://api.polygon.io/v3/snapshot/options/{underlying_asset}"
+                
                 if params:
                     query_string = '&'.join([f"{key}={value}" for key, value in params.items()])
                     endpoint += '?' + query_string
                 endpoint += f"&apiKey={self.api_key}"
-
+                print(endpoint)
                 # Log the API call for debugging purposes
                 logging.debug(f"Fetching option chain data for {underlying_asset} with query: {params}")
 
@@ -1456,6 +1453,101 @@ class PolygonOptions:
             except Exception as e:
                 logging.error(f"An error occurred while fetching the option chain: {e}")
                 
+    async def get_option_chain_all_spx(self, 
+                                    underlying_asset: str, 
+                                    strike_price: float = None, 
+                                    strike_price_lte: float = None, 
+                                    strike_price_gte: float = None, 
+                                    expiration_date: str = None, 
+                                    expiration_date_gte: str = None, 
+                                    expiration_date_lte: str = None, 
+                                    contract_type: str = None, 
+                                    order: str = None, 
+                                    limit: int = 250, 
+                                    sort: str = None, 
+                                    insert: bool = False):
+        """
+        Retrieve all options contracts for a specific underlying asset (ticker symbol) across multiple pages.
+        
+        This function paginates through all available data for a given ticker symbol and applies optional filters like 
+        strike price, expiration date, and contract type. It returns a structured object with the fetched data and 
+        optionally inserts the data into a database.
+
+        :param underlying_asset: The underlying ticker symbol of the option contract (e.g., 'AAPL', 'SPX').
+        :param strike_price: Exact strike price of the option contract.
+        :param strike_price_lte: Strike price less than or equal to a specific value.
+        :param strike_price_gte: Strike price greater than or equal to a specific value.
+        :param expiration_date: Specific expiration date of the contract (YYYY-MM-DD).
+        :param expiration_date_gte: Expiration date on or after a specific date (YYYY-MM-DD).
+        :param expiration_date_lte: Expiration date on or before a specific date (YYYY-MM-DD).
+        :param contract_type: Type of contract ('call' or 'put').
+        :param order: Order results based on the specified field, either 'asc' or 'desc'.
+        :param limit: Maximum number of results per API request (default 250, max allowed by API).
+        :param sort: Field by which to sort the results (e.g., 'expiration_date').
+        :param insert: If True, inserts the fetched data into the database (default False).
+        :return: A UniversalOptionSnapshot object containing option chain data across all pages.
+        
+        :raises ValueError: If an invalid input parameter is provided.
+        :raises Exception: For any failures in data retrieval or database interaction.
+        """
+        try:
+            if not underlying_asset:
+                raise ValueError("Underlying asset ticker symbol must be provided.")
+
+            # Handle special case for index assets (e.g., "I:SPX" for S&P 500 Index)
+            if underlying_asset.startswith("I:"):
+                underlying_asset = underlying_asset.replace("I:", "")
+
+            # Build query parameters
+            params = {
+                'strike_price': strike_price,
+                'strike_price.lte': strike_price_lte,
+                'strike_price.gte': strike_price_gte,
+                'expiration_date': expiration_date,
+                'expiration_date.gte': expiration_date_gte,
+                'expiration_date.lte': expiration_date_lte,
+                'contract_type': contract_type,
+                'order': order,
+                'limit': limit,
+                'sort': sort
+            }
+
+            # Filter out None values
+            params = {key: value for key, value in params.items() if value is not None}
+
+            # Construct the API endpoint and query string
+            endpoint = f"https://api.polygon.io/v3/snapshot/options/{underlying_asset}"
+            
+            if params:
+                query_string = '&'.join([f"{key}={value}" for key, value in params.items()])
+                endpoint += '?' + query_string
+            endpoint += f"&apiKey={self.api_key}"
+            print(endpoint)
+            # Log the API call for debugging purposes
+            logging.debug(f"Fetching option chain data for {underlying_asset} with query: {params}")
+
+            # Fetch the data using asynchronous pagination
+            response_data = await self.paginate_concurrent(endpoint)
+
+            # Parse response data into a structured option snapshot object
+            option_data = UniversalOptionSnapshot(response_data)
+
+            # Insert into the database if specified
+            if insert == True:
+                logging.info("Inserting option chain data into the database.")
+                await self.connect()  # Ensure connection to the database
+                await self.batch_insert_dataframe(option_data.df, table_name='all_options', unique_columns='option_symbol')
+
+            logging.info("Option chain data retrieval successful.")
+            return option_data
+
+        except ValueError as ve:
+            logging.error(f"ValueError occurred: {ve}")
+            
+
+        except Exception as e:
+            logging.error(f"An error occurred while fetching the option chain: {e}")
+            
 
 
 
@@ -1958,15 +2050,7 @@ class PolygonOptions:
         # Fetch records
 
         return chunks
-    async def multi_snapshot_httpx(self, symbol_string: str):
-        async with httpx.AsyncClient() as client:
-            url = f"https://api.polygon.io/v3/snapshot?ticker.any_of={symbol_string}&apiKey={os.environ.get('YOUR_POLYGON_KEY')}&limit=250"
-            resp = await client.get(url)
-            data = resp.json()
-            results = data['results'] if 'results' in data else None
-            if results is not None:
-                data = OptionData(results)
-                return data
+
     async def close(self):
         if self.pool:
             await self.pool.close()
