@@ -43,7 +43,7 @@ class CapitalFlow:
             The data is fetched asynchronously using aiohttp.
     """
 
-    def __init__(self, item):
+    def __init__(self, item, ticker):
         print(item)
         self.superin = float(item['superLargeInflow']) if 'superLargeInflow' in item else None
         self.superout = float(item['superLargeOutflow']) if 'superLargeOutflow' in item else None
@@ -78,6 +78,7 @@ class CapitalFlow:
 
 
         self.data_dict = {
+            'ticker': ticker,
             'superLargeInflow': self.superin,
             'superLargeOutflow': self.superout,
             'superLargeNetFlow': self.supernet,
@@ -109,8 +110,8 @@ class CapitalFlow:
             'retailOutflow': self.retailout,
             'retailOutflowRatio': self.retailoutratio
         }
-        df = pd.DataFrame(self.data_dict, index=[0])
-        self.df= format_large_numbers_in_dataframe(df)
+        self.df = pd.DataFrame(self.data_dict, index=[0])
+        
 
 
 class CapitalFlowHistory:
