@@ -1,8 +1,9 @@
 import pandas as pd
 class CostDistribution:
-    def __init__(self, r):
+    def __init__(self, r, ticker):
         all_data_dicts = []
         datas = r['data']
+        self.all_profit_ratios = []
         for data in datas:
             self.tickerId = data['tickerId']
             self.avgCost = data['avgCost'] if 'avgCost' in data else None
@@ -20,6 +21,7 @@ class CostDistribution:
 
 
             self.data_dict = { 
+                'ticker': ticker,
                 'Avg Cost': self.avgCost,
                 '% Shareholders in Profit': self.closeProfitRatio,
                 'Close': self.close,
@@ -28,6 +30,7 @@ class CostDistribution:
                 
             }
             all_data_dicts.append(self.data_dict)
+            self.all_profit_ratios.append(self.closeProfitRatio)
         try:
             self.df = pd.DataFrame(all_data_dicts, index=[0])
         except Exception as e:
@@ -39,3 +42,33 @@ class CostDistribution:
 
     def __repr__(self):
         return self.__str__()
+
+
+
+class NewCostDist:
+    def __init__(self, data, ticker):
+        self.tickerId = [i.get('tickerId') for i in data]
+        self.avgCost = [float(i.get('avgCost')) for i in data]
+        self.closeProfitRatio = [round(float(i.get('closeProfitRatio'))*100,2) for i in data]
+        self.chip90Start = [i.get('chip90Start') for i in data]
+        self.chip90End = [i.get('chip90End') for i in data]
+        self.chip90Ratio = [i.get('chip90Ratio') for i in data]
+        self.chip70Start = [i.get('chip70Start') for i in data]
+        self.chip70End = [i.get('chip70End') for i in data]
+        self.chip70Ratio = [i.get('chip70Ratio') for i in data]
+        self.close = [float(i.get('close')) for i in data]
+        self.totalShares = [i.get('totalShares') for i in data]
+        self.tradeStamp = [i.get('tradeStamp') for i in data]
+        self.dates = [pd.to_datetime(ts, unit='ms').strftime('%Y-%m-%d') for ts in self.tradeStamp]
+
+
+        self.data_dict = { 
+            'ticker': [ticker] * len(self.close),
+            'date': self.dates,
+            'price': self.close,
+            'profit_ratio': self.closeProfitRatio,
+            'average_cost': self.avgCost
+        }
+
+
+        self.as_dataframe = pd.DataFrame(self.data_dict)
