@@ -6,7 +6,7 @@ import aiohttp
 import asyncio
 load_dotenv()
 
-opts = WebullOptions(database='markets', user='chuck')
+opts = WebullOptions(database='market_data', user='chuck')
 class WebullTrader:
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
