@@ -351,26 +351,37 @@ class WebullTA:
         return total_range != 0 and body_length <= 0.1 * total_range and upper_shadow == 0 and lower_shadow > 2 * body_length
 
 
-    def is_uptrend(self, df: pd.DataFrame, length: int = 5) -> bool:
-        """Check if the dataframe shows an uptrend over the specified length."""
+    def is_uptrend(self, df: pd.DataFrame, length: int =7) -> bool:
+        """
+        Check if the dataframe shows an uptrend over the specified length.
+        
+        An uptrend is defined as consecutive increasing 'Close' values for the given length.
+        The dataframe is assumed to have the most recent candle at index 0.
+        """
         try:
-            for i in range(1, length):
-                if df['Close'].iloc[i] <= df['Close'].iloc[i - 1]:
-                    return False
-            return True
+            if len(df) < length:
+                raise ValueError(f"DataFrame length ({len(df)}) is less than the specified length ({length})")
+            
+            # Since the most recent data is at index 0, we need to reverse the direction of comparison.
+            return (df['Close'].iloc[:length].diff(periods=-1).iloc[:-1] > 0).all()
+
         except Exception as e:
-            print(e)
-            return False
-    def is_downtrend(self, df: pd.DataFrame, length: int = 5) -> bool:
-        """Check if the dataframe shows a downtrend over the specified length."""
+            print(f"Failed - {e}")
+
+    def is_downtrend(self, df: pd.DataFrame, length: int = 7) -> bool:
+        """
+        Check if the dataframe shows a downtrend over the specified length.
+        
+        A downtrend is defined as consecutive decreasing 'Close' values for the given length.
+        """
         try:
-            for i in range(1, length):
-                if df['Close'].iloc[i] >= df['Close'].iloc[i - 1]:
-                    return False
-            return True
+            if len(df) < length:
+                raise ValueError(f"DataFrame length ({len(df)}) is less than the specified length ({length})")
+            
+            # Since the most recent data is at index 0, we need to reverse the direction of comparison.
+            return (df['Close'].iloc[:length].diff(periods=-1).iloc[:-1] < 0).all()
         except Exception as e:
-            print(e)
-            return False
+            print(f"Failed - {e}")
 
     def is_hammer(self,row):
         body_length = abs(row['Close'] - row['Open'])
@@ -528,337 +539,101 @@ class WebullTA:
                 prev_row['Close'] < prev_prev_row['Close'] and curr_row['Close'] < prev_row['Close'])
     
 
-    async def get_ta(self, ticker, headers):
+
+
+    async def get_uptrend_downtrend(self, ticker, interval: str = 'd', length: int = 10, headers=None):
         try:
-            # Dictionary to collect patterns for each interval
-            ticker_patterns = {}
+            # Fetch the data (assuming this returns a DataFrame with the most recent candle at index 0)
+            data = await trading.async_get_td9(ticker=ticker, interval=interval, headers=headers, count=200)
 
-            # Iterate through each interval for the ticker
-            for interval in self.intervals_to_scan:
-                # Fetch the DataFrame asynchronously
-                df = await self.async_get_td9(ticker=ticker, interval=interval, headers=headers)
+            # Initialize columns for uptrend and downtrend
+            data['uptrend'] = False
+            data['downtrend'] = False
 
-                # Call the asynchronous scan_candlestick_patterns function
-                patterns_df = await self.async_scan_candlestick_patterns(df, interval)
+            # Check if there is an uptrend
+            if self.is_uptrend(data, length):
+                data.loc[:length-1, 'uptrend'] = True  # Mark the latest `length` rows as uptrend
 
-                # Since the DataFrame is in ascending order (oldest first), the last row is the most recent data
-                last_row = patterns_df.iloc[-1]
+            # Check if there is a downtrend
+            if self.is_downtrend(data, length):
+                data.loc[:length-1, 'downtrend'] = True  # Mark the latest `length` rows as downtrend
 
-                # Identify patterns that are True
-                pattern_columns = ['hammer', 'inverted_hammer', 'hanging_man', 'shooting_star', 'doji',
-                    'bullish_engulfing', 'bearish_engulfing', 'bullish_harami', 'bearish_harami',
-                    'morning_star', 'evening_star', 'piercing_line', 'dark_cloud_cover',
-                    'three_white_soldiers', 'three_black_crows', 'abandoned_baby',
-                    'rising_three_methods', 'falling_three_methods', 'three_inside_up', 'three_inside_down',
-                    'gravestone_doji', 'butterfly_doji', 'harami_cross', 'tweezer_top', 'tweezer_bottom']
+            # Filter the rows where either 'uptrend' or 'downtrend' is True
+            filtered_data = data[(data['uptrend'] == True) | (data['downtrend'] == True)]
 
-                true_patterns = [pattern for pattern in pattern_columns if last_row[pattern]]
-
-                if true_patterns:
-                    signal = last_row['signal']
-                    # Store the patterns and signal for this interval
-                    ticker_patterns[interval] = {
-                        'patterns': true_patterns,
-                        'signal': signal
-                    }
-
-            # Return the ticker patterns
-            return ticker_patterns
+            # Return the filtered DataFrame with only the columns that are True for trends
+            if not filtered_data.empty:
+                return filtered_data.head(length)
+            else:
+                return None  # Return None if no trend is detected
 
         except Exception as e:
-            print(f"Exception in processing {ticker}: {e}")
+            print(f"{ticker}: {e}")
             return None
         
-    async def detect_ascending_triangle(self, df):
-        """
-        Detects ascending triangle patterns in a stock price DataFrame across multiple timespans.
+    def classify_candle(self,open_value, close_value):
+        if close_value > open_value:
+            return "green"
+        elif close_value < open_value:
+            return "red"
+        else:
+            return "neutral"
 
-        Parameters:
-        - df (pd.DataFrame): DataFrame containing stock data with at least 'Timestamp', 'Close', 'High', 'Low', 'timespan', and 'Ticker' columns.
+    # Function to classify candle colors across all intervals
+    def classify_candle_set(self,opens, closes):
+        return [self.classify_candle(open_val, close_val) for open_val, close_val in zip(opens, closes)]
 
-        Returns:
-        - pd.DataFrame: Original DataFrame with an additional 'AscendingTriangle' column indicating pattern detection.
-        """
-        try:
-            import numpy as np
-            from scipy.signal import find_peaks
+    # Function to classify shapes across rows for one set of rows
+    def classify_shape(self,open_val, high_val, low_val, close_val, color, interval, ticker):
+        body = abs(close_val - open_val)
+        upper_wick = high_val - max(open_val, close_val)
+        lower_wick = min(open_val, close_val) - low_val
+        total_range = high_val - low_val
 
-            # Define parameter mappings based on timespans
-            timespan_params = {
-                'm5':    {'window': 20,  'threshold': 0.005},
-                'm30':   {'window': 25,  'threshold': 0.007},
-                'm60':   {'window': 30,  'threshold': 0.007},
-                'm120':  {'window': 40,  'threshold': 0.008},
-                'm240':  {'window': 50,  'threshold': 0.008},
-                'd':     {'window': 60,  'threshold': 0.009},
-                'w':     {'window': 70,  'threshold': 0.01},
-                'm':     {'window': 80,  'threshold': 0.012}
-            }
+        if total_range == 0:
+            return None  # Skip if there's no valid data
 
-            # Initialize the 'AscendingTriangle' column
-            df['AscendingTriangle'] = False
+        body_percentage = (body / total_range) * 100
+        upper_wick_percentage = (upper_wick / total_range) * 100
+        lower_wick_percentage = (lower_wick / total_range) * 100
 
-            # Ensure the DataFrame is sorted by 'Ticker', 'timespan', and 'Timestamp' in ascending order
-            df = df.sort_values(['Ticker', 'timespan', 'Timestamp']).reset_index(drop=True)
+        if body_percentage < 10 and upper_wick_percentage > 45 and lower_wick_percentage > 45:
+            return f"Doji ({color}) - {ticker} [{interval}]"
+        elif body_percentage > 60 and upper_wick_percentage < 20 and lower_wick_percentage < 20:
+            return f"Long Body ({color}) - {ticker} [{interval}]"
+        elif body_percentage < 30 and lower_wick_percentage > 50:
+            return f"Hammer ({color}) - {ticker} [{interval}]" if color == "green" else f"Hanging Man ({color}) - {ticker} [{interval}]"
+        elif body_percentage < 30 and upper_wick_percentage > 50:
+            return f"Inverted Hammer ({color}) - {ticker} [{interval}]" if color == "green" else f"Shooting Star ({color}) - {ticker} [{interval}]"
+        elif body_percentage < 50 and upper_wick_percentage > 20 and lower_wick_percentage > 20:
+            return f"Spinning Top ({color}) - {ticker} [{interval}]"
+        else:
+            return f"Neutral ({color}) - {ticker} [{interval}]"
 
-            # Group the DataFrame by 'Ticker' and 'timespan'
-            grouped = df.groupby(['Ticker', 'timespan'])
+    # Function to classify candle shapes across all intervals for a given ticker
+    def classify_candle_shapes(self, opens, highs, lows, closes, colors, intervals, ticker):
+        return [self.classify_shape(open_val, high_val, low_val, close_val, color, interval, ticker)
+                for open_val, high_val, low_val, close_val, color, interval in zip(opens, highs, lows, closes, colors, intervals)]
 
-            for (ticker, timespan), group in grouped:
-                params = timespan_params.get(timespan, {'window': 30, 'threshold': 0.005})
-                window = params['window']
-                threshold = params['threshold']
+    # Function to compare two consecutive candles and detect patterns like engulfing and tweezers
+    def compare_candles(self, opens1, closes1, highs1, lows1, colors1, opens2, closes2, highs2, lows2, colors2, intervals, ticker):
+        patterns = []
 
-                if len(group) < window:
-                    continue
+        for open1, close1, high1, low1, color1, open2, close2, high2, low2, color2, interval in zip(opens1, closes1, highs1, lows1, colors1, opens2, closes2, highs2, lows2, colors2, intervals):
+            # Bullish Engulfing
+            if color1 == "red" and color2 == "green" and open2 < close1 and close2 > open1:
+                patterns.append(f"Bullish Engulfing - {ticker} [{interval}]")
 
-                high_prices = group['High'].values
-                low_prices = group['Low'].values
-                indices = group.index
+            # Bearish Engulfing
+            elif color1 == "green" and color2 == "red" and open2 > close1 and close2 < open1:
+                patterns.append(f"Bearish Engulfing - {ticker} [{interval}]")
 
-                # Use the last 'window' data points
-                recent_highs = high_prices[-window:]
-                recent_lows = low_prices[-window:]
-                recent_indices = indices[-window:]
+            # Tweezer Top
+            elif color1 == "green" and color2 == "red" and high1 == high2:
+                patterns.append(f"Tweezer Top - {ticker} [{interval}]")
 
-                # Find peaks in recent_highs (horizontal resistance)
-                peaks, _ = find_peaks(recent_highs, distance=window//5)
-                if len(peaks) < 2:
-                    continue
+            # Tweezer Bottom
+            elif color1 == "red" and color2 == "green" and low1 == low2:
+                patterns.append(f"Tweezer Bottom - {ticker} [{interval}]")
 
-                # Check if peak prices are within threshold (forming horizontal resistance)
-                peak_prices = recent_highs[peaks]
-                resistance_level = np.mean(peak_prices)
-                resistance_diff = np.abs(peak_prices - resistance_level) / resistance_level
-
-                if np.max(resistance_diff) < threshold:
-                    # Find troughs in recent_lows (ascending support)
-                    troughs, _ = find_peaks(-recent_lows, distance=window//5)
-                    if len(troughs) < 2:
-                        continue
-
-                    # Check if troughs are ascending
-                    trough_prices = recent_lows[troughs]
-                    x = np.arange(len(trough_prices))
-                    slope, _ = np.polyfit(x, trough_prices, 1)
-
-                    if slope > 0:
-                        # All conditions met, mark 'AscendingTriangle' at the latest index
-                        df.at[indices[-1], 'AscendingTriangle'] = True
-
-            return df
-        except Exception as e:
-            print(f"Error in detect_ascending_triangle: {e}")
-            return df
-
-    async def detect_rounding_bottom_dynamic(self, df):
-        """
-        Detects rounding bottom patterns in a stock price DataFrame across multiple timespans.
-
-        Parameters:
-        - df (pd.DataFrame): DataFrame containing stock data with at least 'Timestamp', 'Close', 'timespan', and 'Ticker' columns.
-
-        Returns:
-        - pd.DataFrame: Original DataFrame with an additional 'RoundingBottom' column indicating pattern detection.
-        """
-        try:
-            # Define parameter mappings based on timespans
-            timespan_params = {
-                'm5':    {'window': 10,  'threshold': 0.002},  # Short-term patterns with smaller windows
-                'm30':   {'window': 15,  'threshold': 0.003},
-                'm60':   {'window': 20,  'threshold': 0.003},
-                'm120':  {'window': 30,  'threshold': 0.004},
-                'm240':  {'window': 40,  'threshold': 0.004},  # Higher volatility might need larger thresholds
-                'd':     {'window': 50,  'threshold': 0.005},  # Daily chart requires a wider window to capture significant trends
-                'w':     {'window': 60,  'threshold': 0.007},  # Weekly patterns require even wider windows and higher thresholds
-                'm':     {'window': 80,  'threshold': 0.01}    # Monthly charts might need the largest window/threshold
-            }
-
-            # Initialize the 'RoundingBottom' column
-            df['RoundingBottom'] = False
-
-            # Ensure the DataFrame is sorted by 'Ticker', 'timespan', and 'Timestamp' in ascending order
-            df = df.sort_values(['Ticker', 'timespan', 'Timestamp']).reset_index(drop=True)
-
-            # Group the DataFrame by 'Ticker' and 'timespan'
-            grouped = df.groupby(['Ticker', 'timespan'])
-
-            for (ticker, timespan), group in grouped:
-                params = timespan_params.get(timespan, {'window': 20, 'threshold': 0.005})
-                window = params['window']
-                threshold = params['threshold']
-
-                close = group['Close'].values
-                indices = group.index
-
-                # Define the number of candles to confirm the uptrend after the trough
-                trend_confirm_candles = 1  # Mark the second candle after trough
-
-                # Iterate over the group to find troughs
-                # Start from 'window' to len(close) - trend_confirm_candles to avoid index errors
-                for i in range(window, len(close) - trend_confirm_candles):
-                    # Define the current trough candidate
-                    current_trough = close[i]
-
-                    # Define the window for trough comparison
-                    trough_window = close[i - window:i + window + 1]
-
-                    # Check if current_trough is the minimum in the trough_window
-                    if current_trough == min(trough_window):
-                        # Ensure left side is in a downtrend (last ten candles decreasing)
-                        left_trend = close[i - 10:i]
-                        if left_trend[1] < left_trend[0]:
-                            # Ensure right side is in an uptrend (next two candles increasing)
-                            right_trend = close[i + 1:i + trend_confirm_candles + 1]
-                            is_uptrend = all(right_trend[j] > right_trend[j - 1] for j in range(1, len(right_trend)))
-
-                            if is_uptrend:
-                                # Calculate percentage increase from trough to confirmation candle
-                                confirmation_price = close[i + 1]  # second candle after trough
-                                pct_increase = (confirmation_price - current_trough) / current_trough
-
-                                if pct_increase >= threshold:
-                                    # Mark 'RoundingBottom' at confirmation candle
-                                    confirmation_index = i + 2  # first candle after trough
-                                    if confirmation_index < len(close):
-                                        df.at[indices[confirmation_index], 'RoundingBottom'] = True
-
-            return df
-        except Exception as e:
-            print(f"Error in detect_rounding_bottom_dynamic: {e}")
-            return df  # Return the original df in case of error
-
-
-    async def detect_rounding_top_dynamic(self, df):
-        """
-        Detects rounding top patterns in a stock price DataFrame across multiple timespans.
-
-        Parameters:
-        - df (pd.DataFrame): DataFrame containing stock data with at least 'Timestamp', 'Close', 'timespan', and 'Ticker' columns.
-
-        Returns:
-        - pd.DataFrame: Original DataFrame with an additional 'RoundingTop' column indicating pattern detection.
-        """
-        try:
-            # Define parameter mappings based on timespans
-            timespan_params = {
-                'm5':    {'window': 10,  'threshold': 0.002},  # Short-term patterns with smaller windows
-                'm30':   {'window': 15,  'threshold': 0.003},
-                'm60':   {'window': 20,  'threshold': 0.003},
-                'm120':  {'window': 30,  'threshold': 0.004},
-                'm240':  {'window': 40,  'threshold': 0.004},  # Higher volatility might need larger thresholds
-                'd':     {'window': 50,  'threshold': 0.005},  # Daily chart requires a wider window to capture significant trends
-                'w':     {'window': 60,  'threshold': 0.007},  # Weekly patterns require even wider windows and higher thresholds
-                'm':     {'window': 80,  'threshold': 0.01}    # Monthly charts might need the largest window/threshold
-            }
-
-            # Initialize the 'RoundingTop' column
-            df['RoundingTop'] = False
-
-            # Ensure the DataFrame is sorted by 'Ticker', 'timespan', and 'Timestamp' in ascending order
-            df = df.sort_values(['Ticker', 'timespan', 'Timestamp']).reset_index(drop=True)
-
-            # Group the DataFrame by 'Ticker' and 'timespan'
-            grouped = df.groupby(['Ticker', 'timespan'])
-
-            for (ticker, timespan), group in grouped:
-                params = timespan_params.get(timespan, {'window': 20, 'threshold': 0.005})
-                window = params['window']
-                threshold = params['threshold']
-
-                close = group['Close'].values
-                indices = group.index
-
-                # Define the number of candles to confirm the downtrend after the peak
-                trend_confirm_candles = 1  # User wants to mark the second candle after peak
-
-                # Iterate over the group to find peaks
-                # Start from 'window' to len(close) - trend_confirm_candles to avoid index errors
-                for i in range(window, len(close) - trend_confirm_candles):
-                    # Define the current peak candidate
-                    current_peak = close[i]
-
-                    # Define the window for peak comparison
-                    peak_window = close[i - window:i + window + 1]
-
-                    # Check if current_peak is the maximum in the peak_window
-                    if current_peak == max(peak_window):
-                        # Ensure left side is in an uptrend (last ten candles increasing)
-                        left_trend = close[i - 10:i]
-                        if left_trend[1] > left_trend[0]:
-                            # Ensure right side is in a downtrend (next two candles decreasing)
-                            right_trend = close[i + 1:i + trend_confirm_candles + 1]
-                            is_downtrend = all(right_trend[j] < right_trend[j - 1] for j in range(1, len(right_trend)))
-
-                            if is_downtrend:
-                                # Calculate percentage decrease from peak to confirmation candle
-                                confirmation_price = close[i + 1]  # second candle after peak
-                                pct_decrease = (current_peak - confirmation_price) / current_peak
-
-                                if pct_decrease >= threshold:
-                                    # Mark 'RoundingTop' at confirmation candle
-                                    confirmation_index = i + 2 # first candle after peak
-                                    if confirmation_index < len(close):
-                                        df.at[indices[confirmation_index], 'RoundingTop'] = True
-
-            return df
-        except Exception as e:
-            print(f"Error in detect_rounding_top_dynamic: {e}")
-            return df  # Return the original df in case of erro
-        
-
-
-    async def detect_cup_and_handle(self, df):
-        """
-        Detects cup and handle patterns that end at the latest data point.
-
-        Returns:
-        - pd.DataFrame: Original DataFrame with an additional 'CupAndHandle' column.
-        """
-        try:
-            import numpy as np
-            from scipy.signal import find_peaks
-
-            df['CupAndHandle'] = False
-            df = df.sort_values(['Ticker', 'timespan', 'Timestamp']).reset_index(drop=True)
-            grouped = df.groupby(['Ticker', 'timespan'])
-
-            for (ticker, timespan), group in grouped:
-                close_prices = group['Close'].values
-                indices = group.index
-
-                window = 20  # Adjust as needed for different timespans
-
-                if len(close_prices) < 3 * window:
-                    continue
-
-                # Use the last 3*window data points
-                recent_prices = close_prices[-3 * window:]
-                recent_indices = indices[-3 * window:]
-
-                x = np.arange(len(recent_prices))
-
-                # Fit quadratic curve to recent_prices
-                coeffs = np.polyfit(x, recent_prices, 2)
-                a = coeffs[0]
-
-                # Check if the quadratic coefficient is positive (U-shaped cup)
-                if a <= 0:
-                    continue
-
-                # Handle is the last window data points
-                handle_prices = recent_prices[-window:]
-
-                # Check if handle is a small retracement from the right lip of the cup
-                right_lip = recent_prices[-window - 1]
-                min_handle = np.min(handle_prices)
-                retracement = (right_lip - min_handle) / right_lip
-
-                if 0.02 <= retracement <= 0.05:
-                    # All conditions met, mark 'CupAndHandle' at the latest index
-                    df.at[indices[-1], 'CupAndHandle'] = True
-
-            return df
-        except Exception as e:
-            print(f"Error in detect_cup_and_handle: {e}")
-            return df
+        return patterns
\ No newline at end of file
