@@ -16,7 +16,7 @@ pd.set_option('display.max_rows', None)  # None means show all rows
 class yfSDK:
     def __init__(self):
         self.tickers = all_tickers
-        self.db = PolygonDatabase(database='markets', host='localhost', user='chuck', password='fud', port=5432)
+        self.db = PolygonDatabase(database='market_data', host='localhost', user='chuck', password='fud', port=5432)
    
     async def balance_sheet(self, ticker:str, frequency:str='quarterly', pretty:bool=None, as_dict:bool=False):
         """
