@@ -2,9 +2,7 @@ import os
 from dotenv import load_dotenv
 load_dotenv()
 from apis.polygonio.polygon_options import PolygonOptions
-from apis.webull.webull_options import WebullOptions
 import pandas as pd
-wo = WebullOptions(user='chuck', database='charlie', host='localhost', port=5432, password='fud')
 opts = PolygonOptions()
 import asyncpg
 
@@ -16,7 +14,7 @@ async def ticker_autocomp(inter, ticker: str):
         return ["TYPE A TICKER"]
 
     # assuming db_manager is globally defined and connected
-    query = f"""SELECT DISTINCT ticker FROM options_data WHERE ticker LIKE '{ticker}%' LIMIT 24;"""
+    query = f"""SELECT DISTINCT ticker FROM master_all_two WHERE ticker LIKE '{ticker}%' LIMIT 24;"""
     results = await opts.fetch(query)
     
     if not results:
@@ -71,7 +69,7 @@ async def strike_autocomp(inter, strike: str):
     if not strike:
         return ["TYPE A STRIKE"]
         
-    query = f"""SELECT DISTINCT CAST(strike AS text) FROM options_data WHERE ticker = '{selected_ticker}' AND CAST(strike AS text) LIKE '{strike}%' LIMIT 24;"""
+    query = f"""SELECT DISTINCT CAST(strike AS text) FROM master_all_two WHERE ticker = '{selected_ticker}' AND CAST(strike AS text) LIKE '{strike}%' LIMIT 24;"""
     results = await opts.fetch(query)
     if not results:
         return []
@@ -88,14 +86,14 @@ async def expiry_autocomp(inter, expiry: str):
     if not expiry:
         return ["CHOOSE", "AN", "EXPIRY"]
         
-    query = f"""SELECT DISTINCT CAST(expiry AS text) FROM options_data WHERE ticker = '{selected_ticker}' AND CAST(expiry AS text) LIKE '{expiry}%' LIMIT 24;"""
+    query = f"""SELECT DISTINCT CAST(expiry AS text) FROM master_all_two WHERE ticker = '{selected_ticker}' AND CAST(expiry AS text) LIKE '{expiry}%' LIMIT 24;"""
     results = await opts.fetch(query)
     if not results:
         return []
     df = pd.DataFrame(results, columns=['expiry'])
     # Return the symbols
     df['expiry'] = pd.to_datetime(df['expiry'])
-    df['expiry'] = df['expiry'].apply(lambda x: x.strftime('%y%m%d'))
+    df['expiry'] = df['expiry'].apply(lambda x: x.strftime('%Y-%M-%d'))
     return df['expiry'].str.lower().tolist()[:24]
 
 
