@@ -16,7 +16,7 @@ from apis.webull.modal import WebullModal, VolumeAnalysisModal
 from apis.polygonio.polygon_database import PolygonDatabase
 from discord_.bot_menus.pagination import AlertMenus
 from typing import List
-options = WebullOptions(user='chuck', host='localhost', port=5432, password='fud', database='markets')
+options = WebullOptions(user='chuck', host='localhost', port=5432, password='fud', database='market_data')
 
 
 
@@ -24,7 +24,7 @@ options = WebullOptions(user='chuck', host='localhost', port=5432, password='fud
 class DatabaseCOG(commands.Cog):
     def __init__(self, bot):
         self.bot=bot
-        self.db = PolygonDatabase(user='chuck', database='markets', host='localhost', password='fud', port=5432)
+        self.db = PolygonDatabase(user='chuck', database='market_data', host='localhost', password='fud', port=5432)
 
 
 
