@@ -11,7 +11,7 @@ from fudstop.apis.webull.webull_options.webull_options import WebullOptions
 import datetime
 from discord_.bot_menus.pagination import AlertMenus
 
-wb_opts = WebullOptions(database='markets', user='chuck')
+wb_opts = WebullOptions(database='market_data', user='chuck')
 db = PolygonDatabase(user='chuck', database='charlie', host='localhost', port=5432, password='fud')
 opts = PolygonOptions(user='chuck', database='charlie', host='localhost', port=5432, password='fud')
 
