@@ -28,7 +28,7 @@ high_em="<a:_:1047010108930019340>"
 low_em="<a:_:1044658254137008148>"
 close_em="<a:_:1044647558133334126>"
 underlying_price_em="💲"
-sdk = WebullOptions(database='markets', user='chuck')
+sdk = WebullOptions(database='market_data', user='chuck')
 from disnake import TextInputStyle
 class MyModal(disnake.ui.Modal):
     def __init__(self, label, placeholder, name, conn, query, strat=None):
