@@ -3,7 +3,7 @@ import os
 
 import asyncio
 
-poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 
 
 # Creating a database configuration dictionary using os.environ.get for environment variables
