@@ -1,4 +1,4 @@
-from fudstop.apis.webull.webull_screener import WebulScreener
+from fudstop.apis.webull.webull_paper_trader import WebulScreener
 
 
 screener = WebulScreener()
