@@ -32,7 +32,7 @@ high_em="<a:_:1047010108930019340>"
 low_em="<a:_:1044658254137008148>"
 close_em="<a:_:1044647558133334126>"
 underlying_price_em="💲"
-sdk = WebullOptions(database='markets', user='chuck')
+sdk = WebullOptions(database='market_data', user='chuck')
 
 class FilterMenu(disnake.Embed):
     def __init__(self):
