@@ -7,7 +7,7 @@ from dotenv import load_dotenv
 load_dotenv()
 import datetime
 client = OpenAI(api_key=os.environ.get('YOUR_OPENAI_KEY'))
-sdk = WebullOptions(database='markets', user='postgres')
+sdk = WebullOptions(database='market_data', user='postgres')
 def serialize_record(record):
     """Convert asyncpg.Record to a dictionary, handling date serialization."""
     return {key: value.isoformat() if isinstance(value, datetime.date) else value 
